#include "stm8s.h"
#include "sn76489.h"

// Stores the latched channel for subsequent control byte writes.
u8 PSG_LatchedChannel;
// Records whether we're latching a "period" or "amplitude" write.
u8 PSG_LatchedMode;

extern u16 PSG_PendingCycles;
extern u16 PSG_ClockRate;

// Resets the PSG to its default state.
void PSG_DeInit(void) {
	u8 c;
	for (c = 0; c < 4; ++c) {
		PSG_Channels[c].Period = (u16)0;
		PSG_Channels[c].Amplitude = (u8)15;
		PSG_Channels[c].Counter = (u16)0;
	}
	PSG_Output = (u8)0xF;
	PSG_LatchedChannel = (u8)0;
	PSG_LatchedMode = (u8)0;
	PSG_SampleTimer = (u16)0;
	PSG_ShiftRegister = (u16)0x8000;
	PSG_StereoMask = (u8)0xFF;
	PSG_TappedBits = (u16)0x0009;
	PSG_PendingCycles = (u16)0;
	PSG_ShiftRegisterWidth = (u8)16;
	// Set the default clock rate.
	PSG_SetClockRate((u32)3579545);
}

// Sets the PSG's clock rate.
void PSG_SetClockRate(u32 clock) {
	PSG_ClockRate = clock / 256;
}

// Writes a control byte to the emulated PSG.
void PSG_Write(u8 value) {
	if (value & (u8)0x80) {
		// Latch/Data write.
		PSG_LatchedChannel = (u8)(value >> 5) & (u8)0x3;
		if (PSG_LatchedMode = (u8)((value >> 4) & (u8)0x1)) {
			PSG_Channels[PSG_LatchedChannel].Amplitude = value & (u8)0x0F;
		} else {
			PSG_Channels[PSG_LatchedChannel].Period &= (u16)0xFFF0;
			PSG_Channels[PSG_LatchedChannel].Period |= (u16)(value & 0x0F);
			if (PSG_LatchedChannel == 3) PSG_ShiftRegister = (u16)(1 << (PSG_ShiftRegisterWidth - 1));
		}
	} else {
		// Data write.
		if (PSG_LatchedMode) {
			PSG_Channels[PSG_LatchedChannel].Amplitude = value & (u8)0x0F;
		} else {
			PSG_Channels[PSG_LatchedChannel].Period &= (u16)0xF;
			PSG_Channels[PSG_LatchedChannel].Period |= (u16)((value & 0x3F) << 4);
		}
	}
}