#ifndef SN76489_H
#define SN76489_H

// Represents the state of a single output channel in the PSG.
typedef struct {
	volatile u16 Period;
	volatile u8 Amplitude;
	volatile u16 Counter;
} PSG_Channel;

// Stores the state of the four PSG output channels.
extern PSG_Channel PSG_Channels[4];

// Stores the state of each channel's output as an array of bits.
extern volatile u8 PSG_Output;

// Stores whether each channel is enabled or not for the left and right speakers.
extern u8 PSG_StereoMask;

// Decrements every time a sample is generated until it reaches zero - useful to time playback.
extern volatile u16 PSG_SampleTimer;

// Holds the bits of the PSG's linear feedback shift register.
extern volatile u16 PSG_ShiftRegister;

// "Tapped" bit mask used to control the sound of the white noise channel.
extern u16 PSG_TappedBits;

// Width of the PSG shift register. Can only be 16 (default) or 15.
extern u8 PSG_ShiftRegisterWidth;

// Resets the PSG.
void PSG_DeInit(void);

// Writes a control byte to the emulated PSG.
void PSG_Write(u8 value);

// Sets the PSG's clock rate.
void PSG_SetClockRate(u32 clock);

#endif