#include "stm8s.h"
#include "vgm.h"
#include "sn76489.h"
#include "eeprom.h"

u32 VGM_LoopPoint;

// Loads the VGM file.
void VGM_Load(void) {
	// Deinitialise (reset) the PSG.
	PSG_DeInit();
	// Resets the EEPROM and starts reading from it. 
	EEPROM_Init();
	EEPROM_StartReading(0);
	// Read the loop point.
	VGM_LoopPoint = EEPROM_ReadUIntAt(VGM_HEADER_LOOP_OFFSET);
	// Set the PSG's clock rate.
	PSG_SetClockRate(EEPROM_ReadUIntAt(VGM_HEADER_SN76489_CLOCK));
	// VGM 1.10 additions
	if (EEPROM_ReadUIntAt(VGM_HEADER_VERSION) >= 0x110) {
		PSG_TappedBits = EEPROM_ReadUShortAt(VGM_HEADER_SN76489_FEEDBACK);
		PSG_ShiftRegisterWidth = EEPROM_ReadByteAt(VGM_HEADER_SN76489_SHIFT_REGISTER_WIDTH);
	}
	// Start from the start of the file.
	EEPROM_Seek(0x40);
}

// Fetches data from the VGM and uses it to update the PSG; should be called on a loop.
void VGM_Update(void) {
	
	u8 data;
	u16 delay;

	// Top up the EEPROM buffer whilst waiting for the next event in the VGM file.
	while (PSG_SampleTimer > 0) {
		EEPROM_TopUpBuffer();
	}
	// Fetch data from the VGM and process it.
	switch (data = EEPROM_ReadByteBuffered()) {
		case 0x4F:
			PSG_StereoMask = EEPROM_ReadByteBuffered();
			break;
		case 0x50:
			PSG_Write(EEPROM_ReadByteBuffered());
			break;
		case 0x61:
			delay = EEPROM_ReadByteBuffered();
			delay += (256 * EEPROM_ReadByteBuffered());
			PSG_SampleTimer = delay;
			break;
		case 0x62:
			PSG_SampleTimer = 735;
			break;
		case 0x63:
			PSG_SampleTimer = 882;
			break;
		case 0x66:
			if (VGM_LoopPoint) {
				EEPROM_Seek((u16)(VGM_LoopPoint - VGM_HEADER_LOOP_OFFSET));
			} else {
				EEPROM_Seek((u16)0x40);
			}
			break;
		default:
			if (data >= 0x70 && data <= 0x7F) {
				PSG_SampleTimer = data - 0x6F;
			} else if (data >= 0x30 && data < 0x50) {
				// One dummy read.
				EEPROM_ReadByteBuffered();
			} else if (data >= 0x50 && data < 0xC0) {
				// Two dummy reads.
				EEPROM_ReadByteBuffered();
				EEPROM_ReadByteBuffered();
			} else if (data >= 0xC0 && data <= 0xE0) {
				// Three dummy reads.
				EEPROM_ReadByteBuffered();
				EEPROM_ReadByteBuffered();
				EEPROM_ReadByteBuffered();
			} else if (data >= 0xE1) {
				// Four dummy reads.
				EEPROM_ReadByteBuffered();
				EEPROM_ReadByteBuffered();
				EEPROM_ReadByteBuffered();
				EEPROM_ReadByteBuffered();
			}
			break;
	}
}